"""Top-level package for Structured and Interactive Summarization."""

__author__ = """Mélanie Cambus, Marc-Olivier Buob, Fabien Mathieu"""
__email__ = 'fabien.mathieu@normalesup.org'
__version__ = '0.1.0'
